using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.AI;

//アリが利他行動を行い、橋を作成する
public class AntBridgeManager : MonoBehaviour
{
    private bool isChanged = false;

    // キー:antID, 値:antBridgePrefabの辞書
    private Dictionary<int, GameObject> antDict;

    [SerializeField] private GameObject antBridgePrefab = null;
    private GameObject AntBridgePrefab => antBridgePrefab;

    [SerializeField] private float h = 4;
    [SerializeField] private float w = 8;

    private Mesh initMesh;

    void Start()
    {
        antDict = new Dictionary<int, GameObject>();
        initMesh = Instantiate(gameObject.GetComponent<MeshFilter>().sharedMesh);
    }

    void FixedUpdate()
    {
        // 橋を作成するアリに変化があったとき(AddAnt, RemoveAntが呼び出されたとき)
        if(isChanged) {
            // meshの結合

            transform.GetComponent<MeshFilter>().mesh = Instantiate(initMesh);
            MeshFilter[] meshFilters = GetComponentsInChildren<MeshFilter>();
            CombineInstance[] combine = new CombineInstance[meshFilters.Length];

            int i = 0;
            while (i < meshFilters.Length)
            {
                combine[i].mesh = (meshFilters[i]).sharedMesh;
                combine[i].transform = meshFilters[i].transform.localToWorldMatrix;

                i++;
            }
            var combinedMesh = new Mesh();
            combinedMesh.CombineMeshes(combine);
            transform.GetComponent<MeshFilter>().mesh = combinedMesh;
            transform.gameObject.SetActive(true);
            transform.position = new Vector3(0, 0, 0);

            // NavMeshの作成

            GetComponent<NavMeshSurface>().BuildNavMesh();
        }
        isChanged = false;
    }

    // アリからboxを作成し、橋の一部にする
    // antID: 対象とするアリ/ pos: 橋の境界とアリの衝突位置/ angle:　作成するboxの角度/ direction: boxを作成する方向  
    public void AddAnt(int antID, Vector3 pos, float angle, Vector3 direction)
    {
        if(antDict.ContainsKey(antID))
            return;
        GameObject bridge = Instantiate(antBridgePrefab, pos + new Vector3(w * direction.x / 2.0f, -0.5f, h * direction.z / 2.0f), Quaternion.identity);
        bridge.SetActive(true);
        float angle_euler = Util.ConvertToEuler(angle);
        bridge.transform.rotation = Quaternion.AngleAxis(-angle_euler - 90.0f, Vector3.up);
        bridge.transform.parent = gameObject.transform;
        bridge.transform.Find("Mesh").transform.localScale = new Vector3(h, 1.0f, w); 
        antDict[antID] = bridge;
        isChanged = true;
    }

    // 橋になっていたアリを元に戻す
    public void RemoveAnt(int antID)
    {
        if(!antDict.ContainsKey(antID))
            return;
        Destroy(antDict[antID]);
        antDict.Remove(antID);
        isChanged = true;
    }
}
